@@ -15,7 +15,10 @@
  *   along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
+#include "opentims_all.h"
 #include "opentims.cpp"
 #include "scan2inv_ion_mobility_converter.cpp"
 #include "tof2mz_converter.cpp"
 #include "converters.cpp"
+#include "thread_mgr.cpp"
+#include "so_manager.cpp"
