@@ -16,12 +16,15 @@
 import pathlib
 
 try:
+    import libopentims_support
     import opentimspy_cpp
 except ImportError:
     import ctypes
     import pkgutil
     support_lib = pkgutil.get_loader("libopentims_support")
     ctypes.CDLL(support_lib.get_filename(), ctypes.RTLD_GLOBAL)
+    cpp_lib = pkgutil.get_loader("libopentims_cpp")
+    ctypes.CDLL(cpp_lib.get_filename(), ctypes.RTLD_GLOBAL)
     import opentimspy_cpp
 
 
@@ -62,4 +65,19 @@ if bruker_bridge_present:
 def get_module_dir():
     return pathlib.Path(__file__).parent
 
-__version__ = "1.0.8"
+
+from opentimspy.opentims import OpenTIMS
+
+def set_num_threads(n):
+    '''
+    Set the numer of worker threads that OpenTIMS is allowed to use
+    during its calculations. Note: a setting of 1 still means OpenTIMS
+    might spawn a separate worker thread, just that at most 1 thread
+    at a time will be doing heavy computations.
+
+    A setting of 0 means to use all available cores (this is the default
+    behaviour)
+    '''
+    opentimspy_cpp.set_num_threads(n)
+
+__version__ = "1.0.9"
