@@ -22,8 +22,7 @@ NULL
   library.dynam.unload("opentimsr", libpath)
 }
 
-# The order of these matters!!!
-all_columns = c('frame','scan','tof','intensity','mz','dt','rt')
+all_columns = c('frame','scan','tof','intensity','mz','inv_ion_mobility','retention_time')
 
 
 #' TimsTOF data accessor.
@@ -34,6 +33,16 @@ all_columns = c('frame','scan','tof','intensity','mz','dt','rt')
 #' @slot handle Pointer to raw data.
 #' @slot min_frame The index of the minimal frame.
 #' @slot max_frame The index of the miximal frame.
+#' @slot min_scan The minimal scan number. It is assumed to be equal to 1.
+#' @slot max_scan The maximal scan number.
+#' @slot min_intensity The minimal value of intensity. Set to 0, but actually 9 is more sensible.
+#' @slot max_intensity The maximal intensity: the max over values reported in the frames.
+#' @slot min_retention_time The lowest recorded retention time.
+#' @slot max_retention_time The highest recorded retention time.
+#' @slot min_inv_ion_mobility The minimal recorded inverse ion mobility.
+#' @slot max_inv_ion_mobility The maximal recorded inverse ion mobility.
+#' @slot min_mz The minimal recorded mass to charge ratio.
+#' @slot max_mz The maximal recorded mass to charge ratio.
 #' @slot frames A data.frame with information on the frames (contents of the Frames table in the sqlite db).
 #' @slot all_columns Names of available columns.
 #' @export
@@ -42,20 +51,18 @@ setClass('OpenTIMS',
                    handle='externalptr',
                    min_frame='integer',
                    max_frame='integer',
+                   min_scan='integer',
+                   max_scan='integer',
+                   min_intensity='integer',
+                   max_intensity='integer',
+                   min_retention_time='numeric',
+                   max_retention_time='numeric',
+                   min_inv_ion_mobility='numeric',
+                   max_inv_ion_mobility='numeric',
+                   min_mz='numeric',
+                   max_mz='numeric',
                    frames='data.frame',
-                   all_columns='character'),
-         validity = function(object){
-            d.folder = file.exists(object@path.d) 
-            if(!d.folder) print('The folder with data (typically named *.d) does not exist.')
-          
-            d.folder.analysis.tdf = file.exists(file.path(object@path.d, 'analysis.tdf'))
-            if(!d.folder.analysis.tdf) print('The .d folder does not contain the sqlite data-base called "analysis.tdf".')
-          
-            d.folder.analysis.tdf_bin = file.exists(file.path(object@path.d, 'analysis.tdf_bin'))
-            if(!d.folder.analysis.tdf_bin) print('The .d folder does not contain the raw data file called "analysis.tdf_bin".')
-
-            d.folder & d.folder.analysis.tdf & d.folder.analysis.tdf_bin
-         }
+                   all_columns='character')
 )
 
 setMethod('show',
@@ -71,6 +78,11 @@ setMethod('show',
 #' Get the overall number of peaks.
 #'
 #' @param x OpenTIMS data instance.
+#' @examples
+#' \dontrun{
+#' D = OpenTIMS('path/to/your/folder.d')
+#' print(length(D))
+#' }
 setMethod('length', 
           'OpenTIMS', 
           function(x) tdf_no_peaks_total(x@handle))
@@ -78,7 +90,13 @@ setMethod('length',
 #' Get some frames of data.
 #'
 #' @param x OpenTIMS data instance.
-#' @param i An array of nonzero indices to extract. 
+#' @param i An array of nonzero indices to extract.
+#' @examples
+#' \dontrun{
+#' D = OpenTIMS('path/to/your/folder.d')
+#' print(head(D[10]))
+#' print(head(D[10:100]))
+#' }
 setMethod("[", 
           signature(x = "OpenTIMS", i = "ANY"),
           function(x, i){
@@ -95,7 +113,12 @@ setMethod("[",
 #' @param x OpenTIMS data instance.
 #' @param from The first frame to extract.
 #' @param to The last+1 frame to extract. Frame with that number will not get extracted, but some below that number might.
-#' @param by The by
+#' @param by Extract each by-th frame
+#' @examples
+#' \dontrun{
+#' D = OpenTIMS('path/to/your/folder.d')
+#' print(head(range(D, 10,100,3))) # each third frame from 10 to 100.
+#' }
 setMethod("range", 
           "OpenTIMS",
           function(x, from, to, by=1L){ 
@@ -118,11 +141,17 @@ setMethod("range",
 #' @param names Names to extract from the sqlite database.
 #' @return A list of tables.
 #' @export
+#' @examples
+#' \dontrun{
+#' D = OpenTIMS('path/to/your/folder.d')
+#' print(head(table2df(D, "Frames"))) # Extract table "Frames".
+#' }
 table2df = function(opentims, names){
     analysis.tdf = file.path(opentims@path.d, 'analysis.tdf')
     sql_conn = DBI::dbConnect(RSQLite::SQLite(), analysis.tdf)
     tables = lapply(names, function(name) DBI::dbReadTable(sql_conn, name))
     DBI::dbDisconnect(sql_conn)
+    names(tables) = names
     return(tables)
 }
 
@@ -132,6 +161,11 @@ table2df = function(opentims, names){
 #' @param opentims Instance of OpenTIMS
 #' @return Names of tables.
 #' @export
+#' @examples
+#' \dontrun{
+#' D = OpenTIMS('path/to/your/folder.d')
+#' print(tables_names(D)) 
+#' }
 tables_names = function(opentims){
     analysis.tdf = file.path(opentims@path.d, 'analysis.tdf')
     sql_conn = DBI::dbConnect(RSQLite::SQLite(), analysis.tdf)
@@ -157,23 +191,85 @@ OpenTIMS = function(path.d){
     analysis.tdf = file.path(path.d, 'analysis.tdf')
     sql_conn = DBI::dbConnect(RSQLite::SQLite(), analysis.tdf)
     frames = DBI::dbReadTable(sql_conn, 'Frames')
+    GlobalMetadata = DBI::dbReadTable(sql_conn, 'GlobalMetadata')
+    GlobalMetadata = array(GlobalMetadata$Value, dimnames=list(GlobalMetadata$Key))
     DBI::dbDisconnect(sql_conn)
-    handle = tdf_open(path.d)
+    handle = tdf_open(path.d, frames)
+
+    ## Extracting basic info on the limits of reported measurements.
+    ## Does not include TOF index
+
+    # since we must use RSQLite anyway, we don't have to read it from C++
+    # min_frame=as.integer(tdf_min_frame_id(handle)),
+    # max_frame=as.integer(tdf_max_frame_id(handle)),
+    min_frame = as.integer(min(frames$Id)) # just in case
+    max_frame = as.integer(max(frames$Id)) # just in case
+    min_scan = 1L
+    max_scan = as.integer(max(frames$NumScans))
+    min_intensity = 0L
+    max_intensity = max(frames$MaxIntensity)
+    min_retention_time = min(frames$Time)
+    max_retention_time = max(frames$Time)
+    min_inv_ion_mobility = as.numeric(GlobalMetadata['OneOverK0AcqRangeLower'])
+    max_inv_ion_mobility = as.numeric(GlobalMetadata['OneOverK0AcqRangeUpper'])
+    min_mz = as.numeric(GlobalMetadata['MzAcqRangeLower']) # would "float" be 
+    max_mz = as.numeric(GlobalMetadata['MzAcqRangeUpper']) # a bad name to use?
 
     new("OpenTIMS",
         path.d=path.d,
         handle=handle,
-        min_frame=as.integer(tdf_min_frame_id(handle)),
-        max_frame=as.integer(tdf_max_frame_id(handle)),
+        min_frame=min_frame,
+        max_frame=max_frame,
+        min_scan=min_scan,
+        max_scan=max_scan,
+        min_intensity=min_intensity,
+        max_intensity=max_intensity,
+        min_retention_time=min_retention_time,
+        max_retention_time=max_retention_time,
+        min_inv_ion_mobility=min_inv_ion_mobility,
+        max_inv_ion_mobility=max_inv_ion_mobility,
+        min_mz=min_mz,
+        max_mz=max_mz,
         frames=frames,
         all_columns=all_columns)
 }
 
+
+#' Get border values for measurements.
+#'
+#' Get the min-max values of the measured variables (except for TOFs, that would require iteration through data rather than parsing metadata).
+#' 
+#' @param opentims Instance of OpenTIMS.
+#' @return data.frame Limits of individual extracted quantities.
+#' @export
+#' @examples
+#' \dontrun{
+#' D = OpenTIMS('path/to/your/folder.d')
+#' min_max_measurements(D) # this gives a small data-frame with min and max values.
+#' }
+min_max_measurements = function(opentims){
+    data.frame(stat=c('min','max'),
+               frame=c(opentims@min_frame,opentims@max_frame),
+               scan=c(opentims@min_scan, opentims@max_scan),
+               intensity=c(opentims@min_intensity, opentims@max_intensity),
+               retention.time=c(opentims@min_retention_time,
+                                opentims@max_retention_time),
+               inv_ion_mobility=c(opentims@min_inv_ion_mobility,
+                                  opentims@max_inv_ion_mobility),
+               mz=c(opentims@min_mz, opentims@max_mz))
+}
+
+
 #' Get MS1 frame numbers.
 #'
 #' @param opentims Instance of OpenTIMS
 #' @return Numbers of frames corresponding to MS1, i.e. precursor ions.
 #' @export
+#' @examples
+#' \dontrun{
+#' D = OpenTIMS('path/to/your/folder.d')
+#' print(MS1(D)) 
+#' }
 MS1 = function(opentims) opentims@frames$Id[opentims@frames$MsMsType == 0]
 
 
@@ -183,6 +279,11 @@ MS1 = function(opentims) opentims@frames$Id[opentims@frames$MsMsType == 0]
 #' @param ... Parameters passed to head and tail functions.
 #' @importFrom utils head tail
 #' @export
+#' @examples
+#' \dontrun{
+#' D = OpenTIMS('path/to/your/folder.d')
+#' explore.tdf.tables(D) 
+#' }
 explore.tdf.tables = function(opentims, ...){
     for(table_name in tables_names(opentims)){
         print(table_name)
@@ -200,6 +301,11 @@ explore.tdf.tables = function(opentims, ...){
 #' @param opentims Instance of OpenTIMS.
 #' @return Number of peaks in each frame.
 #' @export
+#' @examples
+#' \dontrun{
+#' D = OpenTIMS('path/to/your/folder.d')
+#' print(peaks_per_frame_cnts(D)) 
+#' }
 peaks_per_frame_cnts = function(opentims){
   opentims@frames$NumPeaks
 }
@@ -210,7 +316,12 @@ peaks_per_frame_cnts = function(opentims){
 #' @param opentims Instance of OpenTIMS.
 #' @return Retention times corresponding to each frame.
 #' @export
-rts = function(opentims){
+#' @examples
+#' \dontrun{
+#' D = OpenTIMS('path/to/your/folder.d')
+#' print(retention_times(D)) 
+#' }
+retention_times = function(opentims){
   opentims@frames$Time
 }
 
@@ -218,13 +329,19 @@ rts = function(opentims){
 #' Query for raw data.
 #'
 #' Get the raw data from Bruker's 'tdf_bin' format.
-#' Defaults to both raw data ('frame','scan','tof','intensity') and its tranformations into physical units ('mz','dt','rt').
+#' Defaults to both raw data ('frame','scan','tof','intensity') and its tranformations into physical units ('mz','inv_ion_mobility','retention_time').
 #'
 #' @param opentims Instance of OpenTIMS.
 #' @param frames Vector of frame numbers to extract.
 #' @param columns Vector of columns to extract. Defaults to all columns.
 #' @return data.frame with selected columns.
 #' @export
+#' @examples
+#' \dontrun{
+#' D = OpenTIMS('path/to/your/folder.d')
+#' print(query(D, c(1,20, 53)) # extract all columns
+#' print(query(D, c(1,20, 53), columns=c('scan','intensity')) # only 'scan' and 'intensity'
+#' }
 query = function(opentims,
                  frames,
                  columns=all_columns){
@@ -239,32 +356,43 @@ query = function(opentims,
                            get_tofs = col[3],
                            get_intensities = col[4],
                            get_mzs = col[5],
-                           get_dts = col[6],
-                           get_rts = col[7] )
+                           get_inv_ion_mobilities = col[6],
+                           get_retention_times = col[7] )
 
-  as.data.frame(df)[,columns, drop=F] # What an idiot invented drop=T as default???? 
+  as.data.frame(df)[,columns, drop=F] # What an idiot invented drop=T as default????
 }
 
 
 #' Query for raw data.
 #'
 #' Get the raw data from Bruker's 'tdf_bin' format.
-#' Defaults to both raw data ('frame','scan','tof','intensity') and its tranformations into physical units ('mz','dt','rt').
+#' Defaults to both raw data ('frame','scan','tof','intensity') and its tranformations into physical units ('mz','inv_ion_mobility','retention_time').
 #'
 #' We assume 'from' <= 'to'.
 #'
 #' @param opentims Instance of OpenTIMS.
 #' @param from First frame to extract.
 #' @param to Last frame to extract.
-#' @param step Every step-th frame gets extracted (starting from the first one).
+#' @param by Every by-th frame gets extracted (starting from the first one).
 #' @param columns Vector of columns to extract. Defaults to all columns.
 #' @return data.frame with selected columns.
 #' @export
-query_slice = function(opentims, from=NULL, to=NULL, step=1L, columns=all_columns){
+#' @examples
+#' \dontrun{
+#' D = OpenTIMS('path/to/your/folder.d')
+#' print(query_slice(D, 10, 200, 4)) # extract every fourth frame between 10 and 200. 
+#' print(query_slice(D, 10, 200, 4, columns=c('scan','intensity')) # only 'scan' and 'intensity'
+#' }
+query_slice = function(opentims,
+                       from=NULL,
+                       to=NULL,
+                       by=1,
+                       columns=all_columns){
+  col = opentims@all_columns %in% columns
 
   # Border conditions.
   if(is.null(from)) from = opentims@min_frame
-  if(is.null(to)) to = opentims@max_frame 
+  if(is.null(to)) to = opentims@max_frame
 
   df = tdf_extract_frames_slice( opentims@handle,
                                  from,
@@ -275,8 +403,8 @@ query_slice = function(opentims, from=NULL, to=NULL, step=1L, columns=all_column
                                  get_tofs = col[3],
                                  get_intensities = col[4],
                                  get_mzs = col[5],
-                                 get_dts = col[6],
-                                 get_rts = col[7] )
+                                 get_inv_ion_mobilities = col[6],
+                                 get_retention_times = col[7] )
   as.data.frame(df)[,columns, drop=F] # What an idiot invented drop=T as default????
 }
 
@@ -287,25 +415,30 @@ get_right_frame = function(x,y) ifelse(x < y[1], NA, findInterval(x, y, left.ope
 
 #' Get the retention time for each frame.
 #'
-#' Extract all frames corresponding to retention times inside [min_rt, max_rt] closed borders interval.
+#' Extract all frames corresponding to retention times inside [min_retention_time, max_retention_time] closed borders interval.
 #'
 #' @param opentims Instance of OpenTIMS.
-#' @param min_rt Lower boundry on retention time.
-#' @param max_rt Upper boundry on retention time.
+#' @param min_retention_time Lower boundry on retention time.
+#' @param max_retention_time Upper boundry on retention time.
 #' @param columns Vector of columns to extract. Defaults to all columns.
 #' @return data.frame with selected columns.
 #' @export
+#' @examples
+#' \dontrun{
+#' D = OpenTIMS('path/to/your/folder.d')
+#' print(rt_query(D, 10, 100)) # frames between tenth and a hundreth second of the experiment
+#' }
 rt_query = function(opentims,
-                    min_rt,
-                    max_rt,
-                    columns=c('frame','scan','tof','intensity','mz','dt','rt')){
-  RTS = rts(opentims)
+                    min_retention_time,
+                    max_retention_time,
+                    columns=all_columns){
+  RTS = retention_times(opentims)
 
-  min_frame = get_left_frame(min_rt, RTS)
-  max_frame = get_right_frame(max_rt, RTS)
+  min_frame = get_left_frame(min_retention_time, RTS)
+  max_frame = get_right_frame(max_retention_time, RTS)
 
   if(is.na(min_frame) | is.na(max_frame))
-    stop("The [min_rt,max_rt] interval does not hold any data.")
+    stop("The [min_retention_time,max_retention_time] interval does not hold any data.")
 
   query_slice(opentims,
               from=min_frame,
@@ -327,6 +460,10 @@ rt_query = function(opentims,
 #' @return Path to the output 'timsdata.dll' on Windows and 'libtimsdata.so' on Linux.
 #' @importFrom utils download.file
 #' @export
+#' @examples
+#' \dontrun{
+#' download_bruker_proprietary_code("your/prefered/destination/folder")
+#' }
 download_bruker_proprietary_code = function(
   target.folder, 
   net_url=paste0("https://github.com/MatteoLacki/opentims_bruker_bridge/",
@@ -357,7 +494,7 @@ download_bruker_proprietary_code = function(
 }
 
 
-#' Dynamically link Bruker's DLL to enable tof-mz and scan-dt conversion.
+#' Dynamically link Bruker's DLL to enable tof-mz and scan-inv_ion_mobility conversion.
 #'
 #' By using this function you aggree to terms of license precised in "https://github.com/MatteoLacki/opentims_bruker_bridge".
 #' The conversion, due to independent code-base restrictions, are possible only on Linux and Windows operating systems.
@@ -365,5 +502,10 @@ download_bruker_proprietary_code = function(
 #'
 #' @param path Path to the 'libtimsdata.so' on Linux or 'timsdata.dll' on Windows, as produced by 'download_bruker_proprietary_code'.
 #' @export
+#' @examples
+#' \dontrun{
+#' so_path = download_bruker_proprietary_code("your/prefered/destination/folder")
+#' setup_bruker_so(so_path)
+#' }
 setup_bruker_so = function(path) .setup_bruker_so(path)
 
